package com.brave.service.imp;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.brave.constants.JwtConstants;
import com.brave.constants.TokenConstants;
import com.brave.domain.User;
import com.brave.feign.AuthFeign;
import com.brave.result.Result;
import com.brave.service.AuthService;
import com.brave.utils.JwtUtils;
import com.brave.utils.StringUtils;
import com.brave.utils.TelSmsUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private AuthFeign authFeign;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private HttpServletRequest request;

    @Override
    public Result sendCode(String phone) {
        if (StringUtils.isEmpty(phone)){
            return Result.error("手机号不能为空");
        }

        Result<User> result=authFeign.findUserPhone(phone);
        User data = result.getData();
        if (data==null){
            return Result.error("手机号未注册");
        }

        if (redisTemplate.hasKey(phone)){
            String code = redisTemplate.opsForValue().get(phone);
            String s = TelSmsUtils.sendSms(phone, code, new HashMap<String, String>() {{
                put("code", code);
            }});

            User user = JSONObject.parseObject(s, User.class);
            if ("OK".equals(user.getCode())){
                TelSmsUtils.sendSms(phone,code,new HashMap<String,String>(){{
                    put("code",code);
                }});
            }
            return Result.success(code);
        }


        String code = RandomUtil.randomNumbers(4);
        redisTemplate.opsForValue().set(phone,code,5, TimeUnit.MINUTES);
        String s = TelSmsUtils.sendSms(phone, code, new HashMap<String, String>() {{
            put("code", code);
        }});

        User user = JSONObject.parseObject(s, User.class);
        if ("OK".equals(user.getCode())){
            TelSmsUtils.sendSms(phone,code,new HashMap<String,String>(){{
                put("code",code);
            }});
        }

        return Result.success(code);
    }

    @Override
    public Result<User> codeLogin(User user) {
        if (StringUtils.isAllEmpty(user.getPhone(),user.getCode())){
            return Result.error("手机号不能为空");
        }

        Result<User> result=authFeign.findUserPhone(user.getPhone());
        User data = result.getData();
        if (data==null){
            return Result.error("手机号未注册");
        }

        String code = redisTemplate.opsForValue().get(user.getPhone());
        if (code==null){
            return Result.error("验证码过期");
        }


        if (!code.equals(user.getCode())){
            return Result.error("验证码过期");
        }


        String userkey = UUID.randomUUID().toString().replaceAll("-", "");
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.DETAILS_USER_ID,user.getUserId());
        map.put(JwtConstants.USER_KEY,userkey);
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userkey,JSONObject.toJSONString(user),5,TimeUnit.MINUTES);
        data.setToken(token);
        data.setExpirtTime("5Min");
        return Result.success(data);
    }

    @Override
    public Result<User> info() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        return Result.success(user);
    }
}
