package com.brave.filters;

import com.brave.config.IgnoreWhiteConfig;
import com.brave.utils.GatewayUtils;
import com.brave.constants.TokenConstants;
import com.brave.utils.JwtUtils;
import com.brave.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;


@Component
public class AuthFilter implements GlobalFilter, Ordered {

    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {

        List<String> whites = ignoreWhiteConfig.getWhites();
        for (String white : whites) {
            System.out.println(white);
        }

        ServerHttpRequest request = exchange.getRequest();
        String path = request.getURI().getPath();
        System.out.println(path);
        if (StringUtils.matches(path, whites)) {
            // 白名单请求 放行
            return chain.filter(exchange);
        }

        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        if (StringUtils.isEmpty(token)) {
            return GatewayUtils.errorResponse(exchange, "token不能为空！", HttpStatus.UNAUTHORIZED);
        }
        // token 合法性  如果token 是假的的 方法就会抛出异常
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange, "token不合法！");
        }

        String userKey = JwtUtils.getUserKey(token);
        Boolean hasKey = redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        if (null == hasKey || !hasKey) {
            return GatewayUtils.errorResponse(exchange, "token过期！");
        }

        redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY + userKey, 30, TimeUnit.MINUTES);

        return chain.filter(exchange);
    }


    @Override
    public int getOrder() {
        return 0;
    }
}
