package com.brave.config;

import com.brave.constants.RabbitContants;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
@Log4j2
public class SmsConfig {


    @Autowired
    private RedisTemplate<String,String> redisTemplate;


    @RabbitListener(queuesToDeclare = {@Queue(RabbitContants.SEND_CODE)})
    public void sendCode(String msg, Message message, Channel channel){
        try {
            log.info("登录日志消费者接收消息，消息内容是：{}，开始消费",msg);
            String messageId = message.getMessageProperties().getMessageId();
            Long count = redisTemplate.opsForSet().add(RabbitContants.SEND_CODE, messageId);
            if (1==count){
                log.info("登录日志消费者接收消息，已确认，消息内容是：{}",msg);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
            }else{
                log.info("登录日志消费者接收消息重复，消息内容是：{}",msg);
            }
        }catch (Exception e){
            log.error("登录日志消费者消息异常，消息内容是：{}，异常消息是：{}",msg,e);
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),true);
                log.error("登录日志消费者回退消息异常，消息内容是：{}，异常消息是：{}",msg,e);
            } catch (IOException ex) {
                throw new RuntimeException(ex);
            }
        }
    }
}
